/*
** Copyright (C) 1999 Open Source Telecom Corporation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "answer.h"
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/file.h>
#include <paths.h>
#include <sys/ioctl.h>

static	char *pknames[] = 
{
	"Unknown",
	"V12",
	"Daytona",
	"Premiere",
	"Trans4M",
	"Dsp Module",
	"InLine GT",
	"InLine GTB",
	"InLine EX",
	"InLine ST",
	"PrimeNet",
	"Daytona PCI",
	"Premiere PCI",
	"PrimeNet PCI"
};

PKCARD	pkcard;
VPPORT	vpports[MAX_LINES];
int	vplines = 0;
int	playsize = 0;
char	playbuf[MAX_RECORD * 8000];
int	bufsize;
	
void stop(int signo)
{
	int p;
	VPPORT *port;

	for(p = 0; p < vplines; ++p)
	{
		port = &vpports[p];
		if(port->pid)
			waitpid(port->pid, NULL, 0);
		PK_TRUNK_SetEventHandler(port->hCTI, 0, 0);
		PK_TRUNK_OnHook(port->hCTI);
	}

	printf("card %d\n", pkcard.id);
	PIKA_StopDriver(pkcard.hBoard);
	PK_DSP_StopDriver(pkcard.id);
	syslog(LOG_INFO, "%s stopped %d", pkcard.name, signo);
	closelog();
	exit(0);
}

static void start(int board)
{
	int nBoards = PIKA_NumberOfBoards();
	int r1, r2, r3;
	sigset_t sigs, msigs;
	TResourceHandle hTrk;
	TResourceHandle hDsp;
	VPPORT	*port;

	r1 = open("/var/spool/vm/prompt.ul", O_RDONLY);
	if(r1 > -1)
	{
		playsize = read(r1, playbuf, sizeof(playbuf));
		close(r1);
		syslog(LOG_INFO, "loaded %d bytes", playsize);
	}

	/* Since pika uses pthread, we must make sure the callback
	   thread does not inherit or respond to any controlling
	   signals.  We do this by masking the main thread's signals
	   before the pika driver has a chance to start callback.
	*/

	sigemptyset(&sigs);
	sigaddset(&sigs, SIGINT);
	sigaddset(&sigs, SIGTERM);
	sigaddset(&sigs, SIGQUIT);
	pthread_sigmask(SIG_BLOCK, &sigs, &msigs);

	/* Now it's safe to start the driver...
	 */

	--board;
	if(nBoards < 1)
	{
		syslog(LOG_EMERG, "no boards found");
		exit(-1);
	}

	if(board < 0 || board >= nBoards)
	{
		syslog(LOG_EMERG, "invalid board id");
		exit(-1);
	}
	
	pkcard.type = PIKA_BoardType(board) & ~0x1000000;
	if(pkcard.type < sizeof(pknames) / sizeof(char *))
		strcpy(pkcard.name, pknames[pkcard.type]);
	else
		strcpy(pkcard.name, "invalid");
	
	syslog(LOG_INFO, "%s initializing...", pkcard.name);
	pkcard.hBoard = PIKA_InitializeBoard(board);
	if(pkcard.hBoard)
	{
		r1 = PK_MVIP_Initialize(board);
		r2 = PK_DSP_Initialize(board);
		syslog(LOG_INFO, "%s initialized <%d,%d>", pkcard.name, r1, r2);
	}
	else
	{
		syslog(LOG_EMERG, "%s failed", pkcard.name);
		exit(-1);
	}
	pkcard.id = board;
	pkcard.dsp = PK_DSP_NumberOfDevices(board);
	printf("dsp count %d\n", pkcard.dsp);
	hDsp = PK_DSP_GetDeviceHandle(board, 0);
	if(!hDsp)
	{
		syslog(LOG_CRIT, "no dsp resources");
		stop(-1);
	}

	while((hTrk = PIKA_SeizePortHandleOfType(pkcard.hBoard, PK_LS_PORT)))
	{
		port = &vpports[vplines];
		port->fd = -1;
		port->pid = 0;
		port->hPort = hTrk;
		port->hCTI = PIKA_GetResourceFromPort(hTrk);
		port->hDSP = PK_DSP_SeizePort(hDsp, PK_VP_RESOURCES);
		port->hTIMER = PK_TIMER_Create();
		sprintf(port->name, "%s(%d)", pkcard.name, vplines);

		if(!port->hDSP)
		{
			syslog(LOG_CRIT, "%s dsp exhausted", port->name);
			break;
		}

		port->connected = PK_FALSE;

		PK_DSP_SetEventHandler(port->hDSP, (PK_U32)port, (pkevent_t)portEvents);
		r1 = PK_MVIP_FullDuplexConnect(port->hCTI, port->hDSP);
		r2 = PIKA_AssociatePort(port->hPort, port->hDSP);
		PIKA_SetEventHandler(port->hPort, (PK_U32)port, (pkevent_t)portEvents);
		PIKA_SetLSParams(port->hPort, NULL, 3000, 8000, PK_TRUE, 3000);
		PK_TRUNK_SetEventHandler(port->hCTI, (PK_U32)port, (pkevent_t)portEvents);
		PK_TIMER_SetEventHandler(port->hTIMER, (PK_U32)port, (pkevent_t)portEvents);
//		r3 = PK_VP_IncludeTone(port->hDSP, PK_ATD_CALL_PROGRESS_TONE);
//		r3 += PK_VP_EnableToneDetection(port->hDSP);
		/* r3 += PK_VP_EnableSpeechDetection(port->hDSP); */
		r3 = PK_AUDIO_SetFormat(port->hDSP, PK_COMPOUND_MU_LAW | PK_SAMPLING_RATE_8_KHz);
		if(!vplines)
		{
			bufsize = PK_AUDIO_BufferSize(port->hDSP);
			syslog(LOG_INFO, "audio buffer size %d", bufsize);
		}
		PK_AUDIO_InputSetGain(port->hDSP, PK_AUDIO_INPUT_DEFAULT_GAIN + 10);
		PK_AUDIO_OutputSetGain(port->hDSP, PK_AUDIO_OUTPUT_DEFAULT_GAIN + 10);
		PK_TRUNK_CallOfferDetection(port->hCTI, PK_TRUE);
		PK_TRUNK_DisconnectDetection(port->hCTI, PK_TRUE);
		syslog(LOG_INFO, "%s started <%d,%d,%d>", port->name, r1, r2, r3);
		if(++vplines >= MAX_LINES)
			break;
	}	

	pthread_sigmask(SIG_SETMASK, &msigs, NULL);
	signal(SIGINT, stop);
	signal(SIGTERM, stop);
	signal(SIGQUIT, stop);
}		

int main(int argc, char **argv)
{
	VPPORT	*port;
	int	p;	
	int	rc;

	if(argc > 1)
	{
		close(0);
		close(1);
		close(2);
		signal(SIGTTOU, SIG_IGN);
		signal(SIGTTIN, SIG_IGN);
		signal(SIGTSTP, SIG_IGN);
		p = fork();
		if(p)
			exit(0);

		setpgid(0, getpid());
		p = open(_PATH_TTY, O_RDWR);
		if(p)
		{
			ioctl(p, TIOCNOTTY, NULL);
			close(p);
		}
		
		openlog("pkzanswer", LOG_NDELAY, LOG_DAEMON);
	}
	else
		openlog("pkzanswer", LOG_NDELAY | LOG_PERROR, LOG_DAEMON);

	start(1);
	for(;;)
	{
		for(p = 0; p < vplines; ++p)
		{
			port = &vpports[p];		
			if(port->pid)
			{
				rc = waitpid(port->pid, NULL, 0);
				if(rc == port->pid)
					port->pid = 0;
			}
		}	
		sleep(5);
	}

	stop(0);
}

